import QtQuick 2.8
import QtQuick.Layouts 1.1
import QtQuick.Controls 2.5
import org.kde.plasma.plasmoid 2.0
import org.kde.plasma.core 2.1 as PlasmaCore
import org.kde.plasma.components 2.0 as PlasmaComponents

Item {
    id: root
    PlasmaCore.DataSource {
        id: executable
        engine: "executable"
        interval:500
        connectedSources: []
        property var callbacks: ({})
        onNewData: {
            var stdout = data["stdout"]

            if (callbacks[sourceName] !== undefined) {
                callbacks[sourceName](stdout);
            }

            exited(sourceName, stdout)
            disconnectSource(sourceName) // exec finished
        }

        function exec(cmd, onNewDataCallback) {
            if (onNewDataCallback !== undefined){
                callbacks[cmd] = onNewDataCallback
            }
            connectSource(cmd)
        }
        signal exited(string sourceName, string stdout)
    }

    Plasmoid.preferredRepresentation: Plasmoid.compactRepresentation
    Plasmoid.fullRepresentation: Item {
        id: fullRoot

        readonly property double iwSize: units.gridUnit * 12.6 // item width
        readonly property double shSize: .15 // separator height

        // config var
        readonly property string aboutThisComputerCMD: "kcmshell5 kcm_about-distro"
        readonly property string systemPreferencesCMD:"systemsettings5"
        readonly property string appStoreCMD: "/usr/bin/octopi"
        readonly property string printerCMD: "kcmshell5 kcm_printer_manager"
        readonly property string userCMD: "kcmshell5 kcm_users"
        readonly property string displayCMD: "kcmshell5 kcm_kscreen"
        readonly property string shutDownCMD:"qdbus org.kde.ksmserver /KSMServer logout 1 0 0"

        Layout.preferredWidth: iwSize
        Layout.preferredHeight: aboutThisComputerItem.height * 11 // not the best way to code..



        ColumnLayout {
            id: columm
            anchors.fill: parent
            spacing: 0

            ListDelegate {
                id: aboutThisComputerItem
                text: i18n(" About This System")
                Image {
                    source: "help-info.png"
                    anchors.left:parent.left
                    anchors.bottom:parent.bottom
                    width:22
                    height:22
                }
                onClicked: {
                    executable.exec(aboutThisComputerCMD); // cmd exec
                }
            }

            MenuSeparator {
                id: s1
                padding: 0
                topPadding: 5
                bottomPadding: 3
                contentItem: Rectangle {
                    implicitWidth: iwSize
                    implicitHeight: shSize
                    color: "gray"
                }
            }

            ListDelegate {
                id: systemPreferencesItem
                text: i18n(" System Preferences")
                Image {
                    source: "settings.png"
                    width:24
                    height:24
                    anchors.left:parent.left
                    anchors.bottom:parent.bottom
                }
                onClicked: {
                    executable.exec(systemPreferencesCMD); // cmd exec
                }
            }

            ListDelegate {
                id: appStoreItem
                text: i18n(" App Store")
                Image {
                    source: "software-store.png"
                    anchors.left:parent.left
                    anchors.bottom:parent.bottom
                    sourceSize.width: 16
                    sourceSize.height: 16
                }
                onClicked: {
                    executable.exec(appStoreCMD); // cmd exec
                }
            }

            MenuSeparator {
                id: s2
                padding: 0
                topPadding: 5
                bottomPadding: 5
                contentItem: Rectangle {
                    implicitWidth: iwSize
                    implicitHeight: shSize
                    color: "gray"
                }
            }

            ListDelegate {
                id: filemgr
                text: i18n(" Files")
                Image {
                    source: "files.png"
                    width:14
                    height:14
                    anchors.left:parent.left
                    anchors.bottom:parent.bottom
                }
                onClicked: {
                    executable.exec("dolphin");
                }
            }

            ListDelegate {
                id: terminal
                text: i18n(" Terminal")
                Image {
                    source: "terminal.png"
                    width:14
                    height:14
                    anchors.left:parent.left
                    anchors.bottom:parent.bottom
                }
                onClicked: {
                    executable.exec("konsole");
                }
            }

            ListDelegate {
                id: editor
                text: i18n(" Editor")
                Image {
                    source: "text.png"
                    width:14
                    height:14
                    anchors.left:parent.left
                    anchors.bottom:parent.bottom
                }
                onClicked: {
                    executable.exec("kate");
                }
            }

            MenuSeparator {
                id: s3
                padding: 0
                topPadding: 5
                bottomPadding: 5
                contentItem: Rectangle {
                    implicitWidth: iwSize
                    implicitHeight: shSize
                    color: "gray"
                }
            }

            ListDelegate {
                id: shutdownItem
                text: i18n(" User Settings")
                Image {
                    source: "system-users.png"
                    anchors.left:parent.left
                    anchors.bottom:parent.bottom
                    sourceSize.width: 20
                    sourceSize.height: 20
                }
                onClicked: {
                    executable.exec(userCMD);
                }
            }

            ListDelegate {
                id: printerItem
                text: i18n(" Printer Settings")
                Image {
                    source: "printer.png"
                    width:20
                    height:20
                    anchors.left:parent.left
                    anchors.bottom:parent.bottom
                }
                onClicked: {
                    executable.exec(printerCMD);
                }
            }

            ListDelegate {
                id: displaytItem
                text: i18n(" Display Settings")
                Image {
                    source: "computer.png"
                    width:20
                    height:20
                    anchors.left:parent.left
                    anchors.bottom:parent.bottom
                }
                onClicked: {
                    executable.exec(displayCMD);
                }
            }

            ListDelegate {
                id: shutDownItem
                text: i18n(" Shut Down")
                Image {
                    source: "system-shutdown-panel.png"
                    width:20
                    height:20
                    anchors.left:parent.left
                    anchors.bottom:parent.bottom
                }
                onClicked: {
                    executable.exec(shutDownCMD);
                    
                }
            }
        }
    }
}
